<?php

/**
 * A time period is a list of times during various days that are
 * considered to be "valid" times for notifications and service
 * checks. It consists of time ranges for each day of the week
 * that "rotate" once the week has come to an end. Different types
 * of exceptions to the normal weekly time are supported, including:
 * specific weekdays, days of generic months, days of specific months,
 * and calendar dates.
 */
class timeperiod_nacoma_Model extends naemon_nacoma_Model
{
	/** object type */
	const TYPE = 'timeperiod';

	public $properties = array (
		'topic' => 'timeperiod_name',
		'var_index' => array (
			'timeperiod_name' => 'string',
			'alias' => 'string',
			'exclude' => 'mselect',
			'sunday' => 'string',
			'monday' => 'string',
			'tuesday' => 'string',
			'wednesday' => 'string',
			'thursday' => 'string',
			'friday' => 'string',
			'saturday' => 'string',
			'file_id' => 'select',
			'register' => 'bool'
		),
		'widget' => array (
			'register' => 'hidden'
		),
		'required' => array (
			'timeperiod_name',
			'alias',
			'file_id'
		),
		'default' => array (
			'file_id' => 'etc/timeperiods.cfg'
		),
		'special' => array (
			'file_id' => 'FILE',
			'exclude' => 'timeperiod'
		)
	);

	/**
	 * Return the mayi resource for the object
	 * @see object_nacoma_Model::mayi_resource()
	 */
	public function mayi_resource() {
		return "monitor.monitoring.timeperiods";
	}

	/**
	 * Validates the correctness of a timeperiod entry for
	 * a specific day
	 * @param $value string: Timeperiod minute-range specification
	 * @return True if valid, false if not.
	 */
	function validate_timeperiod_day($value)
	{
		# shortcircuit out of nul strings
		if (!strlen($value))
		  return true;

		if (preg_match('/[^0-9-:,]/', $value))
		  return false;

		$period = explode(',', $value);
		foreach ($period as $tp) {
			$times = explode('-', $tp);
			if (count($times) !== 2)
			  return false;

			$foo = array();
			foreach ($times as $t) {
				$ary = explode(':', $t);
				if (count($ary) !== 2)
				  return false;

				$h = $ary[0];
				$m = $ary[1];
				$foo[] = $h * 60 + $m;
				if ($m > 59 || $m < 0 || $h > 24 || $h < 0)
				  return false;
			}
			# make sure timerange is moving forwards
			if ($foo[0] > $foo[1]) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Makes sure the object is safe to write to database, and
	 * that Nagios can later load the timeperiod in such a way
	 * that it does what the user expects of it.
	 * @return True on success, false on errors.
	 */
	function validate_object()
	{
		parent::validate_object();

		$err_msg = "Illegal timerange";
		$tp_days = array('monday', 'tuesday', 'wednesday', 'thursday',
						 'friday', 'saturday', 'sunday');
		foreach ($tp_days as $day) {

			# a timeperiod can be without any or all of the days
			# without it causing any problems so if one isn't set,
			# we ignore it.
			if (!isset($this->obj[$day]))
			  continue;

			$this->obj[$day] = trim($this->obj[$day]);
			if (!$this->validate_timeperiod_day($this->obj[$day]))
				$this->validation_error($day, $err_msg);
		}

		if (is_array($this->custom_vars)) {
			foreach ($this->custom_vars as $day => $value) {
				if (!$this->validate_timeperiod_day($value))
					$this->validation_error($day, $err_msg);
			}
		}
		return empty($this->validation_error_list);
	}

	public function normalize($obj = false, $abort_on_error=true)
	{
		if ($obj === false)
			$obj = $this->obj;

		$props = $this->get_properties();
		$v_index = $props['var_index'];
		$spec = $props['special'];

		foreach ($obj as $k => $v) {
			if (!isset($v_index[$k])) {
				// Probably one of them weird, custom time ranges
				$this->custom_vars[$k] = $v;
				unset($obj[$k]);
				continue;
			}
		}
		parent::normalize($obj, $abort_on_error);
	}
}
