<?php

class nacoma_db {
	static $instance = false;

	/**
	    Fetches the underlying PDO object for the nacoma DB.  Connects if
	    necessary, and throws a PDOException if connection fails.
	*/
	static function db()
	{
		if (self::$instance !== false) {
			return self::$instance;
		}

		$attr = array (
			PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
			PDO::ATTR_AUTOCOMMIT => true,
			PDO::ATTR_PERSISTENT => false
		);
		$dbconf = Kohana::config('database.nacoma.connection');

		// Convert ninja config format to nacoma config format
		$c = array(
			'dsn' => $dbconf['type'].':host='.$dbconf['host'].';dbname='.$dbconf['database'],
			'user' => $dbconf['user'],
			'passwd' => $dbconf['pass']
		);

		$pdo = new PDO( $c['dsn'], @$c['user'], @$c['passwd'], $attr );

		foreach ( $attr as $k => $v ) {
			$pdo->setAttribute( $k, $v );
		}

		self::$instance = $pdo;

	    return self::$instance;
	}

	static function print_error(PDOException $ex)
	{
		bug('There was a SQL error', true, $ex->getMessage(), $ex->getTrace());
	}

	/**
	 * WARNING!
	 *
	 * This *might* give you the last ID for the table that you inserted a new row
	 * with an unspecified ID into just now, but that's only if you got lucky!
	 */
	static function last_insert_id($tbl)
	{
	    $db = self::db();
	    return $db ? $db->lastInsertId($tbl) : false;
	}


	/**
	 * Using prepared statements is in theory faster than execute. But even more
	 * important, using self::exec_query with non-ascii characters is sometimes
	 * buggier.
	 */
	static function prepare_statement($query, $show_message=true) {
		global $DEBUG;
		if(empty($query)) return(false);
		$db = self::db();
		try {
			$stmt = $db->prepare($query);
		} catch (PDOException $ex) {
			if ($show_message)
				self::print_error($ex);
			throw $ex;
		}
		return($stmt);
	}

	/**
	 * Executes an SQL query, and returns the PDO result object
	 *
	 * If the caller handles an exception properly itself, it can set $show_message
	 * to false to avoid disturbing the user.
	 */
	static function exec_query($query, $show_message=true) {
		global $DEBUG;
		if(empty($query)) return(false);
		$db = self::db();
		try {
			$result = $db->query($query);
		} catch (PDOException $ex) {
			if ($show_message)
				self::print_error($ex);
			throw $ex;
		}
		return($result);
	}


	/**
	   Returns $string escaped as SQL and surrounded by single-quotes.
	   If $string is-not-a string then it might be returned as-is.
	 */
	static function quote_string($string)
	{
		$db = self::db();
		return $db->quote($string);
	}

	/**
	 Equivalent to $result->fetch().
	*/
	static function fetch_row($result, $fetch_style=PDO::FETCH_BOTH)
	{
	    return $result->fetch($fetch_style);
	}

	/**
	 Copies the results of the next cursor position into a associative array object
	 and returns that object. Properties with SQL NULL or empty string values
	 are not copied!
	*/
	static function fetch_object($result) {
		global $DEBUG;
		if(empty($result)) {
	            if($DEBUG) echo "SQL ERROR: self::fetch_object() called with empty resource\n";
	            return(false);
		}
		$row = $result->fetch(PDO::FETCH_ASSOC);
		if ($row === false) {
	            return false;
		}
		foreach($row as $k => $v) {
			if(empty($v) && $v !== 0 && $v !== '0') continue;
			$ret[$k] = $v;
		}
		return($ret);
	}


	/**
	 Similar to self::fetch_object(), but returns an array of all result objects.
	*/
	static function fetch_result($result) {
		global $DEBUG;

		if(!$result) {
			if($DEBUG) echo "SQL ERROR: self::fetch_result() called with empty resource\n";
			return(false);
		}
		$ret = array();
		$id = false;
		$i = 0;
		while($row = $result->fetch(PDO::FETCH_ASSOC)) {
			$i++;
			if(isset($row['id'])) {
				$id = $row['id'];
				unset($row['id']);
			}
			else $id = $i;
			if($DEBUG) {
				if(isset($ret[$id])) echo "SQL WARNING: About to overwrite row in self::fetch_result()\n";
			}
			foreach($row as $k => $v) {
				if(empty($v) && $v !== 0 && $v !== '0') continue;
				$ret[$id][$k] = $v;
			}
		}
		return empty($ret) ? false : $ret;
	}


	/**
 	 * Counts rows affected by last query
 	 *
 	 * @param $ln PDO = null
	 * @return int
	 */
	static function affected_rows($ln = NULL) {
	    if (!$ln)
	        $ln = self::db();
	    return $ln->rowCount();
	}
}
